/*
/////////////
/// GetDP ///
/////////////

// File: allInOne.pro
// AUTHOR: Frederic Trillaud <ftrillaudp@gmail.com>
// UNIVERSITY: UNAM
// INSTITUTE: IINGEN
// DATE (english): 05/02/2013
// UNITS: SI otherwise indicated

// INTRODUCTION: Problem definition and its resolution up the post processing of data in a single pro file.
                 getdp allInOneGauge -msh superGenerator.msh -solve resolution -pos postProcessing
*/


///////////////////////////
/// EXTERNAL PARAMETERS ///
///////////////////////////

Include "../ParFiles/geometricParameters.par";
Include "../ParFiles/electricParameters.par";


///////////////
/// REGIONS ///
///////////////

Group
{
/// Regions (per geometries) ///
statorSupport = Region[24];
surroundings = Region[25];
infiniteBoundary = Region[26];
boundarySurface = Region[27];
rotor = Region[28];
rotorSupport = Region[29];
airGap = Region[30];
/// Stator L1 phase:
inwardRadialL1 = Region[31];
innerAzimuthalL1 = Region[1];
outwardRadialL1 = Region[2];
outerAzimuthalL1 = Region[3];
/// Stator L2 phase:
inwardRadialL2 = Region[4];
innerAzimuthalL2 = Region[5];
outwardRadialL2 = Region[6];
outerAzimuthalL2 = Region[7];
/// Stator L3 phase:
inwardRadialL3 = Region[8];
innerAzimuthalL3 = Region[9];
outwardRadialL3 = Region[10];
outerAzimuthalL3 = Region[11];
/// Stator U3 phase:
inwardRadialU3 = Region[12];
innerAzimuthalU3 = Region[13];
outwardRadialU3 = Region[14];
outerAzimuthalU3 = Region[15];
/// Stator U2 phase:
inwardRadialU2 = Region[16];
innerAzimuthalU2 = Region[17];
outwardRadialU2 = Region[18];
outerAzimuthalU2 = Region[19];
/// Stator U1 phase:
inwardRadialU1 = Region[20];
innerAzimuthalU1 = Region[21];
outwardRadialU1 = Region[22];
outerAzimuthalU1 = Region[23];

/// Domains (set of regions) ///
/// Inductor domain:
inductorDomain = Region[{inwardRadialL1, innerAzimuthalL1, outwardRadialL1, outerAzimuthalL1, inwardRadialL2, innerAzimuthalL2, outwardRadialL2, outerAzimuthalL2, inwardRadialL3, innerAzimuthalL3, outwardRadialL3, outerAzimuthalL3, inwardRadialU3, innerAzimuthalU3, outwardRadialU3, outerAzimuthalU3, inwardRadialU2, innerAzimuthalU2, outwardRadialU2, outerAzimuthalU2, inwardRadialU1, innerAzimuthalU1, outwardRadialU1, outerAzimuthalU1}];
/// Boundary domain:
infiniteBoundaryDomain = Region[{infiniteBoundary}];
/// paramagnetic domain;
paramagneticDomain = Region[{rotor, rotorSupport, statorSupport, airGap, surroundings}];
/// Electromagnetic solution domain:
solutionDomain = Region[{inductorDomain, paramagneticDomain, infiniteBoundaryDomain}];
}


////////////////
/// FUNCTION ///
////////////////

Function
{
// Permeability of vacuum:
muVacuum = 4*Pi*1e-7;
nu[solutionDomain] = 1./muVacuum;

// Engineering Current density ///
/// Lower stator phases (L)
currentDensity = 100/(4e-3*0.2e-3);
JeL1 = currentDensity; JeL2 = currentDensity; JeL3 = currentDensity;
/// Stator L1 phase:
Je[inwardRadialL1] = (-1)*JeL1*Unit[Vector[X[], Y[], Z[]]];
Je[innerAzimuthalL1] = JeL1*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]]; // Counter-clockwise
Je[outwardRadialL1] = JeL1*Unit[Vector[X[], Y[], Z[]]];
Je[outerAzimuthalL1] = JeL1*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
/// Stator L2 phase:
Je[inwardRadialL2] = (-1)*JeL2*Unit[Vector[X[], Y[], Z[]]];
Je[innerAzimuthalL2] = JeL2*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
Je[outwardRadialL2] = JeL2*Unit[Vector[X[], Y[], Z[]]];
Je[outerAzimuthalL2] = JeL2*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
/// Stator L3 phase:
Je[inwardRadialL3] = (-1)*JeL3*Unit[Vector[X[], Y[], Z[]]];
Je[innerAzimuthalL3] = JeL3*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
Je[outwardRadialL3] = JeL3*Unit[Vector[X[], Y[], Z[]]];
Je[outerAzimuthalL3] = JeL3*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
/// Upper stator phases (U) ///
JeU1 = currentDensity; JeU2 = currentDensity; JeU3 = currentDensity;
/// Stator U3 phase:
Je[inwardRadialU3] = (-1)*JeU3*Unit[Vector[X[], Y[], Z[]]];
Je[innerAzimuthalU3] = JeU3*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
Je[outwardRadialU3] = JeU3*Unit[Vector[X[], Y[], Z[]]];
Je[outerAzimuthalU3] = JeU3*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
/// Stator U2 phase:
Je[inwardRadialU2] = (-1)*JeU2*Unit[Vector[X[], Y[], Z[]]];
Je[innerAzimuthalU2] = JeU2*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
Je[outwardRadialU2] = JeU2*Unit[Vector[X[], Y[], Z[]]];
Je[outerAzimuthalU2] = JeU2*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
/// Stator U1 phase:
Je[inwardRadialU1] = (-1)*JeU1*Unit[Vector[X[], Y[], Z[]]];
Je[innerAzimuthalU1] = JeU1*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
Je[outwardRadialU1] = JeU1*Unit[Vector[X[], Y[], Z[]]];
Je[outerAzimuthalU1] = JeU1*Unit[Vector[Cos[Pi/2]*X[]-Sin[Pi/2]*Y[], Sin[Pi/2]*X[]+Cos[Pi/2]*Y[], Z[]]];
}


///////////////////
/// INTEGRATION ///
///////////////////

Integration
{
 {
  Name advancedIntegration;
  Case
  {
   {
    Type Gauss;
    Case
    {
     {
      GeoElement Point;
      NumberOfPoints 1;
     }
     {
      GeoElement Line;
      NumberOfPoints 4;
     }
     {
      GeoElement Triangle;
      NumberOfPoints 6;
     }
     {
      GeoElement Quadrangle;
      NumberOfPoints 6;
     }
     {
      GeoElement Tetrahedron;
      NumberOfPoints 15;
     }
     {
      GeoElement Hexahedron;
      NumberOfPoints 15;
     }
     {
      GeoElement Pyramid;
      NumberOfPoints 15;
     }
     {
      GeoElement Prism;
      NumberOfPoints 15;
     }
    }
   }
  }
 }
 {
  Name basicIntegration;
	Case
	{
   {
	  Type Gauss;
	  Case
	  {
     {
      GeoElement Line;
      NumberOfPoints 6;
     } 
	   {
	    GeoElement Triangle;
	    NumberOfPoints 6;
	   }
		 {
		  GeoElement Quadrangle;
		  NumberOfPoints 4;
		 }
		}                                                 
   }
	}
 }
}


////////////////
/// JACOBIAN ///
////////////////

Group
{
 DefineGroup[infiniteBoundaryDomain];
 DefineVariable[innerBoundaryRadius, outerBoundaryRadius];
}

Jacobian
{
 {Name jacobianVolumeRegion;
  Case
  {
   {
    Region infiniteBoundaryDomain;
    Jacobian VolSphShell{innerBoundaryRadius, outerBoundaryRadius};
   }
   {
    Region All;
    Jacobian Vol;
   }
  }
 }
}


///////////////////
/// CONSTRAINTS ///
///////////////////

Constraint
{
 {
  Name dirichletBoundaryCondition;
  Type Assign;
  Case
  {
   {
    Region boundarySurface;
    Value 0.;
   }
  }
 }
 {
  Name gaugeCondition;
  Type Assign;
  Case
  {
   {
    Region solutionDomain;
    SubRegion boundarySurface;
    Value 0.;
   }
  }
 }
}


//////////////////////
/// FUNCTION SPACE ///
//////////////////////

FunctionSpace
{
 { // Potential vector
  Name potentialVectorFunctionSpace;
  Type Form1;
  BasisFunction
  {
   {
    Name aEdge;
    NameOfCoef circulationOnEdge;
    Function BF_Edge;
    Support solutionDomain;
    Entity EdgesOf[All];
   }
  }
  Constraint
  {
   {
    NameOfCoef circulationOnEdge;
    EntityType EdgesOfTreeIn;
    EntitySubType StartingOn;
    NameOfConstraint gaugeCondition;
   }
   {
    NameOfCoef circulationOnEdge;
    EntityType EdgesOf;
    NameOfConstraint dirichletBoundaryCondition;
   }
  }
 }    
}


///////////////////
/// FORMULATION ///
///////////////////

Formulation
{
 {
  Name potentialVectorFormulation;
  Type FemEquation;
  Quantity
  {
   {
    Name A;
    Type Local;
    NameOfSpace potentialVectorFunctionSpace;
   }
  }
  Equation
  {
   Galerkin // magnetic solution domain (entire domain)
   { 
    [nu[]*Dof{Curl A}, {Curl A}]; // d in 2D and Curl in 3D 
    In solutionDomain;
    Jacobian jacobianVolumeRegion;
    Integration advancedIntegration;
   }
   Galerkin // Inductor domain (conductors)
   {
    [-Je[] , {A}];
    In inductorDomain;
    Jacobian jacobianVolumeRegion;
    Integration advancedIntegration;
   }
  }
 }
}


//////////////////
/// RESOLUTION ///
//////////////////

Resolution
{
 {
  Name resolution; // to be called: getdp main -msh ....msh -solve resolution
  System
  {
   { 
    Name potentialVector;
    NameOfFormulation potentialVectorFormulation;
   }
  }
  Operation
  {
   InitSolution[potentialVector];
   Generate[potentialVector];
   Solve[potentialVector];
   SaveSolution[potentialVector];
  }
 }
}


//////////////////////
/// POSTPROCESSING ///
//////////////////////

PostProcessing
{
 {
  Name postProcessing;
  NameOfFormulation potentialVectorFormulation;
  Quantity
  {
   {Name A; Value{Local{[{A}]; In solutionDomain; Jacobian jacobianVolumeRegion;}}}
   {Name AMod; Value{Local{[Norm[{A}]]; In solutionDomain; Jacobian jacobianVolumeRegion;}}}
   {Name H; Value{Local{[nu[]*{Curl A}]; In solutionDomain; Jacobian jacobianVolumeRegion;}}}
   {Name HMod; Value{Local{[Norm[nu[]*{Curl A}]]; In solutionDomain; Jacobian jacobianVolumeRegion;}}}
   {Name B; Value{Local{[{Curl A}]; In solutionDomain; Jacobian jacobianVolumeRegion;}}}
   {Name BMod; Value{Local{[Norm[{Curl A}]]; In solutionDomain; Jacobian jacobianVolumeRegion;}}}
   {Name J; Value{Local{[Je[]] ; In inductorDomain; Jacobian jacobianVolumeRegion;}}}
   {Name JMod; Value{Local{[Norm[Je[]]]; In inductorDomain; Jacobian jacobianVolumeRegion;}}}
///   {Name EMagB; Value{Integral {[(2.*Pi/2.)*nu[]*SquNorm[{d A}]]; In solutionDomain; Jacobian jacobianVolumeRegion; Integration basicIntegration;}}}
///   {Name EMagAJ; Value{Integral {[(2.*Pi/2.)*(({A}*{Je}))]; In inductorDomain; Jacobian jacobianVolumeRegion; Integration basicIntegration;}}}
  }
 }
}

PostOperation
{
 {
  Name postTreatment;
  NameOfPostProcessing postProcessing;
  Operation
  {
   Print[A, OnElementsOf rotorSupport, File "../ResFiles/potentialVector.pos"];
   Print[AMod, OnElementsOf rotorSupport, File "../ResFiles/moduleOfMagneticPotentialVector.pos"];
   Print[B, OnElementsOf rotorSupport, File "../ResFiles/magneticFluxDensity.pos"];
   Print[BMod, OnElementsOf rotorSupport, File "../ResFiles/moduleOfMagneticFluxDensity.pos"];
   Print[BMod, OnPoint {0, 0., 0.}, Format Table, File "../ResFiles/maximumModuleOfMagneticFluxDensity.dat"];
   Print[H, OnElementsOf rotorSupport, File "../ResFiles/magneticField.pos"];
   Print[HMod, OnElementsOf rotorSupport, File "../ResFiles/moduleOfMagneticField.pos"];
   Print[J, OnElementsOf inductorDomain, File "../ResFiles/currentDensity.pos"];
   Print[JMod, OnElementsOf inductorDomain, File "../ResFiles/moduleOfCurrentDensity.pos"];
///   Print[EMagB[magneticSolutionDomain], OnGlobal, Format Table, File "../ResFiles/magneticEnergyB.dat"];
///   Print[EMagAJ[magneticSolutionDomain], OnGlobal, Format Table, File "../ResFiles/magneticEnergyAJ.dat"];
  }
 }
}

